from typing import List
from pydantic import BaseModel
from .Name import Name

class ServiceDescription(BaseModel):
    serviceId: str
    serviceType: str
    code: str
    names: List[Name]
    operatingOrgCodes: List[str]
    serviceCategory: str
    description: str
    astraDescription: str


from pydantic import BaseModel
from .LocationContactAndAddress import LocationContactAndAddress
from typing import Optional

class OriginLocation(BaseModel):
    locationContactAndAddress: LocationContactAndAddress
    locationId: Optional[str]


from pydantic import BaseModel
from typing import List

from .TrackingId import TrackingId

from .OperationalDetail import CompletedPackageDetailOperationalDetail

class CompletedPackageDetail(BaseModel):
    sequenceNumber: int
    trackingIds: List[TrackingId]
    groupNumber: int
    signatureOption: str
    operationalDetail: CompletedPackageDetailOperationalDetail


from pydantic import BaseModel
from typing import List

from .MasterTrackingId import MasterTrackingId

from .ServiceDescription import ServiceDescription

from .CompletedEtdDetail import CompletedEtdDetail

from .DocumentRequirements import DocumentRequirements

from .CompletedPackageDetail import CompletedPackageDetail

from .ShipmentDocument import ShipmentDocument

from .OperationalDetail import OperationalDetail


class CompletedShipmentDetail(BaseModel):
    usDomestic: bool
    carrierCode: str
    masterTrackingId: MasterTrackingId
    serviceDescription: ServiceDescription
    packagingDescription: str
    operationalDetail: OperationalDetail
    completedPackageDetails: List[CompletedPackageDetail]
    documentRequirements: DocumentRequirements
    completedEtdDetail: CompletedEtdDetail
    shipmentDocuments: List[ShipmentDocument]


from pydantic import BaseModel
from typing import Optional


class GenerationDetail(BaseModel):
    type: str
    minimumCopiesRequired: int
    letterhead: Optional[str] = None
    electronicSignature: Optional[str] = None


from dataclasses import dataclass
from enum import Enum
from typing import Optional
from pydantic import BaseModel


class TinType(str, Enum):
    Personal_National: str = "PERSONAL_NATIONAL"
    Personal_State: str = "PERSONAL_STATE"
    Federal: str = "FEDERAL"
    Business_National: str = "BUSINESS_NATIONAL"
    Business_State: str = "BUSINESS_STATE"
    Business_Union: str = "BUSINESS_UNION"
    
class Tin(BaseModel):
    number: Optional[str]
    tinType: Optional[TinType]
    usage: Optional[str]
    effectiveDate: Optional[str]
    expirationDate: Optional[str]    
    
    class Config:
        use_enum_values = True
        validate_all = True


from pydantic import BaseModel
from typing import List, Optional

class ScanLocation(BaseModel):
    streetLines: Optional[List[str]]
    city: Optional[str] = None
    stateOrProvinceCode: Optional[str] = None
    postalCode: Optional[str] = None
    countryCode: Optional[str] = None
    residential: bool
    countryName: Optional[str] = None


from typing import List
from pydantic import BaseModel

from .AccountNumber import AccountNumber

from .Address import Address
from .Contact import Contact
from .Tin import Tin

class SoldTo(BaseModel):
    address: Address
    contact: Contact
    tins: List[Tin]
    accountNumber: AccountNumber
    




from pydantic import BaseModel


class TotalDeclaredValue(BaseModel):
    """
    Args:
        `amount` (`float`): Total Declared Value Amount. Example: 12.45
        `currency` (`str`): This is the currency code for the amount. Example: USD
    """
    amount: float
    currency: str


from typing import List
from pydantic import BaseModel, validator


class Address(BaseModel):
    streetLines: List[str]
    city: str
    countryCode: str
    
    stateOrProvinceCode: str = ""
    postalCode: str = ""
    residential: bool = True
    
    @validator("streetLines", allow_reuse=True)
    def street_lines_validate(cls, value: List[str]):
        for line in value:
            if len(line) > 35:
                raise ValueError(
                    "Street Line Max Length 35!"
                )
            else:
                if len(line) == 0:
                    value.remove(line)
        return value
    
    @validator("city", allow_reuse=True)
    def city_validate(cls, value: str):
        if len(value) > 35:
            raise ValueError(
                "City Max Length 35!"
            )
        return value
    
    @validator("postalCode", allow_reuse=True)
    def postal_code_validate(cls, value: str):
        if len(value) > 10:
            raise ValueError(
                "Postal Code Max Length 10!"
            )
            
        return value
    
    @validator("countryCode", allow_reuse=True)
    def country_code_validate(cls, value: str):
        if len(value) > 2:
            raise ValueError(
                "Country Code Max Length 2!"
            )
        return value
            
    


from .AccountNumber import AccountNumber
from .Address import Address
from .Contact import Contact
from .Recipient import Recipient
from .Request import Request, LabelResponseOptions, MergeLabelDocOption, ProcessingOptionType, RequestedShipment, ShipAction
from .RequestedShipment import RequestedShipment, CommercialInvoice, CommercialInvoiceDetail, Commodity, CustomerImageUsage, CustomsClearanceDetail, CustomsValue, Dimensions, Disposition, LabelPrintingOrientation, LabelSpecification, LabelStockType, LabelRotation, LabelImageType, DutiesPayment, DocumentFormat, EtdDetail, Origin, PaymentType, PackagingType, Payor, PickUpType, RequestedPackageLineItem, ResponsibleParty, ShippingDocumentSpecification, ShipmentSpecialServices, ShippingChargesPayment, TotalCustomsValue, TotalDeclaredValue, Weight, UnitPrice, Unit
from .Shipper import Shipper
from .SoldTo import SoldTo
from .Tin import Tin, TinType
from .TotalDeclaredValue import TotalDeclaredValue



from typing import List
from pydantic import BaseModel

from .OperationalInsturaction import OperationalInstruction

from .Barcode import Barcodes

class OperationalDetail(BaseModel):
    ursaPrefixCode: str
    ursaSuffixCode: str
    originLocationId: str
    originLocationNumber: int
    originServiceArea: str
    destinationLocationId: str
    destinationLocationNumber: int
    destinationServiceArea: str
    destinationLocationStateOrProvinceCode: str
    deliveryDate: str
    deliveryDay: str
    commitDate: str
    commitDay: str
    ineligibleForMoneyBackGuarantee: bool
    astraPlannedServiceLevel: str
    astraDescription: str
    postalCode: str
    stateOrProvinceCode: str
    countryCode: str
    airportId: str
    serviceCode: str
    packagingCode: str
    publishedDeliveryTime: str
    scac: str
    

class CompletedPackageDetailOperationalDetail(BaseModel):
    barcodes: Barcodes
    astraHandlingText: str
    operationalInstructions: List[OperationalInstruction]
    




from dataclasses import dataclass
from enum import Enum
from typing import Optional
from pydantic import BaseModel

from .AccountNumber import AccountNumber

from .RequestedShipment import RequestedShipment


class LabelResponseOptions(str, Enum):
    """
    Label – Indicates request is for encoded bytecode.\n
    Url_Only – Indicates label URL request
    """
    Label: str = "LABEL"
    Url_Only: str = "URL_ONLY"


class MergeLabelDocOption(str, Enum): 
    Labels_Only: str = "LABELS_ONLY"
    NONE: str = "NONE"
    Labels_And_Docs: str = "LABELS_AND_DOCS"


class ShipAction(str, Enum):
    Confirm = "CONFIRM"
    Transfer = "TRANSFER"


class ProcessingOptionType(str, Enum):
    Synchronous_Only: str = "SYNCHRONOUS_ONLY"
    Allow_Asynchronous: str = "ALLOW_ASYNCHRONOUS"
    


class Request(BaseModel):
    labelResponseOptions: LabelResponseOptions
    mergeLabelDocOption: MergeLabelDocOption = MergeLabelDocOption.Labels_Only
    accountNumber: AccountNumber
    shipAction: Optional[ShipAction]
    processingOptionType: ProcessingOptionType = ProcessingOptionType.Synchronous_Only
    oneLabelAtTime: bool = False
    requestedShipment: RequestedShipment

    _Comment_: str = ""
    
    class Config:
        use_enum_values = True
        validate_all = True
    


import datetime
import json
import os
from dataclasses import dataclass
from requests_oauthlib import OAuth2Session
import requests
from FedexSDK.models import Request
from FedexSDK.models.label.Response import LabelResponse
from FedexSDK.models.tracking.TrackResult import TrackResponse

@dataclass
class APIType:
    Production: str = "Production"
    Test: str = "Test"

@dataclass
class ImageType:
    Letter_Head: str = "LETTER_HEAD"
    Signature: str = "SIGNATURE"

class FedExSDK:

    def __init__(self, session: OAuth2Session) -> None:
        if not isinstance(session, OAuth2Session):
            raise Exception("Please Requests Session")
        
        self.session = session
        self.headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {session.access_token}"
        }
        
        self.track_url = "https://apis.fedex.com/track/v1"
        self.ship_url = "https://apis.fedex.com/ship/v1"
    def create_shipment(self, request: Request):
        res = self.session.post(f"{self.ship_url}/shipments", json=request.dict(), headers=self.headers)        
        return LabelResponse(**res.json())
        
        with open(f"{now_date.strftime('%d_%m_%Y_tracking_numbers.txt')}", "a") as f:
            f.write(
                f"{res.json()['output']['transactionShipments'][0]['shipmentDocuments'][0]['trackingNumber']}:{request.requestedShipment.recipients[0].contact.personName}\n"
            )
        
        json.dump(res.json(), open(output_json_path, "w", encoding="utf-8"), ensure_ascii=False)

    
    def track_shipment_by_tracking_number(self, tracking_number: str, ship_date_begin: str, ship_date_end: str):
        
        data = {
            "includeDetailedScans": True,
            "trackingInfo": [
                {
                    "trackingNumberInfo": {
                    "trackingNumber": tracking_number,
                    },
                    "shipDateBegin": ship_date_begin,
                    "shipDateEnd": ship_date_end
                }
            ]
        }
        res = self.session.post(f"{self.track_url}/trackingnumbers", json=data, headers=self.headers)
        return TrackResponse(**res.json())
    
    def add_image(self, reference_id: str, image_name: str, image_index: str, image_type: ImageType, image_path: str):
        name, ext = os.path.splitext(image_path)
        data = {
            "document": {
                "referenceId": reference_id,
                "name": image_name,
                "contentType": f"image/{ext}",
                "meta":{
                    "imageType": image_type,
                    "imageIndex": image_index
                }
            },
            "rules":{
                "workflowName":"LetterheadSignature"
            }
        }
        files = {
            "attachment": open(image_path, "rb").read()
        }
        url = "https://documentapi.prod.fedex.com/documents/v1/lhsimages/upload"
        
        headers = self.headers
        headers["Content-Type"] = "multipart/form-data"
        
        res = self.session.post(url, data=data, headers=headers, files=files)
        
    @classmethod
    def authorize(cls, client_id: str, client_secret: str, api_type: APIType = APIType.Test) -> OAuth2Session:
        if api_type == APIType.Production:
            url = "https://apis.fedex.com/oauth/token"
        elif api_type == APIType.Test:
            url = "https://apis.sandbox.fedex.com/oauth/token"
        else:
            raise ValueError("Please Correct APIType!")
        headers = {
            "Content-Type": "application/x-www-form-urlencoded"
        }
        
        data = {
            "grant_type": "client_credentials",
            "client_id": client_id,
            "client_secret": client_secret
        }

        response = requests.request("POST", url, data=data, headers=headers)
        return OAuth2Session(client_id=client_id, token=response.json())


from pydantic import BaseModel
from typing import Dict, Any, Optional

class Window(BaseModel):
    ends: Optional[str]


class StandardTransitTimeWindow(BaseModel):
    window: Window


class EstimatedDeliveryTimeWindow(BaseModel):
    window: Dict[str, Any]


from dataclasses import dataclass
from enum import Enum
from typing import List, Optional
from pydantic import BaseModel

from .AccountNumber import AccountNumber

from .Contact import Contact

from .Address import Address

from .Recipient import Recipient

from .SoldTo import SoldTo

from .Shipper import Shipper

from .TotalDeclaredValue import TotalDeclaredValue



class PickUpType(str, Enum):
    Contact_Fedex_To_Schedule: str = "CONTACT_FEDEX_TO_SCHEDULE"
    Dropoff_At_Fedex_Location: str = "DROPOFF_AT_FEDEX_LOCATION"
    Use_Schduled_Pickup: str = "USE_SCHEDULED_PICKUP"
    

class PackagingType(str, Enum):
    Your_Packaging: str = "YOUR_PACKAGING"
    Envelope: str = "FEDEX_ENVELOPE"
    Box: str = "FEDEX_BOX"
    Small_Box: str = "FEDEX_SMALL_BOX"
    Medium_Box: str = "FEDEX_MEDIUM_BOX"
    Large_Box: str = "FEDEX_LARGE_BOX"
    Extra_Large_Box: str = "FEDEX_EXTRA_LARGE_BOX"
    Ten_KG_Box: str = "FEDEX_10KG_BOX"
    Twenty_Five_KG_Box: str = "FEDEX_25KG_BOX"
    PAK: str = "FEDEX_PAK"
    TUBE: str = "FEDEX_TUBE"


class Origin(BaseModel):
    address: Address
    contact: Contact


class PaymentType(str, Enum):
    Sender: str = "SENDER"
    Recipient: str = "RECIPIENT"
    Third_Party: str = "THIRD_PARTY"
    Collect: str = "COLLECT"
    


class ResponsibleParty(BaseModel):
    address: Optional[Address]
    contact: Optional[Contact]
    accountNumber: AccountNumber


class Payor(BaseModel):
    responsibleParty: ResponsibleParty
    
    
class ShippingChargesPayment(BaseModel):
    paymentType: PaymentType = PaymentType.Sender
    payor: Optional[Payor]
    
    class Config:
        use_enum_values = True
        validate_all = True

class Unit(str, Enum):
    Cm: str = "CM"
    Inch: str = "IN"
    
class Dimensions(BaseModel):
    length: int
    width: int
    height: int
    units: Unit = Unit.Cm
    
class Weight(BaseModel):
    units: str
    value: str
    
    
class RequestedPackageLineItem(BaseModel):
    _Comment_: str
    groupPackageCount: Optional[int]
    dimensions: Optional[Dimensions]
    weight: Weight


class LabelImageType(str, Enum):
    ZPLII: str = "ZPLII"
    EPL2: str = "EPL2"
    PDF: str = "PDF"
    PNG: str = "PNG"



class LabelPrintingOrientation(str, Enum):
    BOTTOM_EDGE_OF_TEXT_FIRST: str = "BOTTOM_EDGE_OF_TEXT_FIRST"
    TOP_EDGE_OF_TEXT_FIRST: str = "TOP_EDGE_OF_TEXT_FIRST"


class LabelRotation(str, Enum):
    LEFT: str = "LEFT"
    RIGHT: str = "RIGHT"
    UPSIDE_DOWN: str = "UPSIDE_DOWN"
    NONE: str = "NONE"


class LabelStockType(str, Enum):
    PAPER_4X6: str = "PAPER_4X6"
    PAPER_4X675: str = "PAPER_4X675"
    PAPER_4X8: str = "PAPER_4X8"
    PAPER_4X9: str = "PAPER_4X9"
    PAPER_7X475: str = "PAPER_7X475"
    PAPER_85X11_BOTTOM_HALF_LABEL: str = "PAPER_85X11_BOTTOM_HALF_LABEL"
    PAPER_85X11_TOP_HALF_LABEL: str = "PAPER_85X11_TOP_HALF_LABEL"
    PAPER_LETTER: str = "PAPER_LETTER"
    STOCK_4X675_LEADING_DOC_TAB: str = "STOCK_4X675_LEADING_DOC_TAB"
    STOCK_4X8: str = "STOCK_4X8"
    STOCK_4X9_LEADING_DOC_TAB: str = "STOCK_4X9_LEADING_DOC_TAB"
    STOCK_4X6: str = "STOCK_4X6"
    STOCK_4X675_TRAILING_DOC_TAB: str = "STOCK_4X675_TRAILING_DOC_TAB"
    STOCK_4X9_TRAILING_DOC_TAB: str = "STOCK_4X9_TRAILING_DOC_TAB"
    STOCK_4X9: str = "STOCK_4X9"
    STOCK_4X85_TRAILING_DOC_TAB: str = "STOCK_4X85_TRAILING_DOC_TAB"
    STOCK_4X105_TRAILING_DOC_TAB: str = "STOCK_4X105_TRAILING_DOC_TAB"

class LabelSpecification(BaseModel):
    imageType: LabelImageType = LabelImageType.ZPLII
    labelStockType: LabelStockType = LabelStockType.PAPER_4X6
    labelPrintingOrientation: Optional[LabelPrintingOrientation]
    returnedDispositionDetail: Optional[bool]
    labelRotation: Optional[LabelRotation]
    
    class Config:
        use_enum_values = True
        validate_all = True
        


class DutiesPayment(BaseModel):
    paymentType: str


class CommercialInvoice(BaseModel):
    termsOfSale: str
    
class TotalCustomsValue(BaseModel):
    amount: str
    currency: str
    
class UnitPrice(BaseModel):
    amount: str
    currency: str


class CustomsValue(BaseModel):
    amount: str
    currency: str

class Commodity(BaseModel):
    description: str
    countryOfManufacture: str
    harmonizedCode: str
    weight: Weight
    quantity: str
    quantityUnits: str
    unitPrice: UnitPrice
    customsValue: CustomsValue

class CustomsClearanceDetail(BaseModel):
    dutiesPayment: DutiesPayment
    commercialInvoice: CommercialInvoice
    isDocumentOnly: bool
    totalCustomsValue: TotalCustomsValue
    _Comment_: str
    commodities: List[Commodity]

class Disposition(BaseModel):
    dispositionType: str

class DocumentFormat(BaseModel):
    stockType: str
    dispositions: List[Disposition]
    docType: str


class CustomerImageUsage(BaseModel):
    id: str
    type: str
    providedImageType: str

class CommercialInvoiceDetail(BaseModel):
    customerImageUsages: List[CustomerImageUsage]
    documentFormat: DocumentFormat

class ShippingDocumentSpecification(BaseModel):
    shippingDocumentTypes: List[str]
    commercialInvoiceDetail: CommercialInvoiceDetail
    


class EtdDetail(BaseModel):
    requestedDocumentTypes: List[str]

class ShipmentSpecialServices(BaseModel):
    specialServiceTypes: List[str]
    etdDetail: EtdDetail



class EmailAggregationType(str, Enum):
    Per_Package = "PER_PACKAGE"
    Per_Shipment = "PER_SHIPMENT"


class EmailNotificationRecipientType(str, Enum):
    Broker = "BROKER"
    Other = "OTHER"
    Recipient = "RECIPIENT"
    Shipper = "SHIPPER"
    Third_Party = "THIRD_PARTY"


class NotificationFormatType(str, Enum):
    HTML = "HTML"
    Text = "TEXT"

class NotificationEventType(str, Enum):
    ON_DELIVERY = "ON_DELIVERY"
    ON_EXCEPTION = "ON_EXCEPTION"
    ON_SHIPMENT = "ON_SHIPMENT"
    ON_TENDER = "ON_TENDER"
    ON_ESTIMATED_DELIVERY = "ON_ESTIMATED_DELIVERY"
    ON_LABEL = "ON_LABEL"
    ON_PICKUP_DRIVER_ARRIVED = "ON_PICKUP_DRIVER_ARRIVED"
    ON_PICKUP_DRIVER_ASSIGNED = "ON_PICKUP_DRIVER_ASSIGNED"
    ON_PICKUP_DRIVER_DEPARTED = "ON_PICKUP_DRIVER_DEPARTED"
    ON_PICKUP_DRIVER_EN_ROUTE = "ON_PICKUP_DRIVER_EN_ROUTE"

class EmailNotificationRecipient(BaseModel):
    name: Optional[str]
    emailNotificationRecipientType: EmailNotificationRecipientType
    emailAddress: str
    notificationFormatType: Optional[NotificationFormatType]
    notificationType: str = "EMAIL"
    locale: str = "tr_TR"
    notificationEventType: List[NotificationEventType]
    
    class Config:
        use_enum_values = True
        validate_all = True
        
    
    
class EmailNotificationDetail(BaseModel):
    aggregationType: EmailAggregationType = EmailAggregationType.Per_Package
    emailNotificationRecipients: List[EmailNotificationRecipient]
    personalMessage: str
    
    class Config:
        use_enum_values = True
        validate_all = True

class RequestedShipment(BaseModel):
    """
    Args:
        `shipDatestamp` (`Optional[str]`): Ship Date. Format: Format [YYYY-MM-DD]
        
        `totalDeclaredValue` (`Optional[TotalDeclaredValue]`): Sum of all declared values of all packages in a shipment

        
    """
    shipDatestamp: Optional[str]
    blockInsightVisibility: bool = False
    
    totalDeclaredValue: Optional[TotalDeclaredValue]
    shipper: Shipper
    soldTo: Optional[SoldTo]
    shipmentSpecialServices: ShipmentSpecialServices
    recipients: List[Recipient]
    recipientLocationNumber: str = ""
    pickupType: PickUpType = PickUpType.Contact_Fedex_To_Schedule
    serviceType: str = "FEDEX_INTERNATIONAL_PRIORITY"
    packagingType: PackagingType = PackagingType.Your_Packaging
    totalWeight: Optional[float]
    origin: Optional[Origin]
    shippingChargesPayment: ShippingChargesPayment
    totalPackageCount: Optional[int]
    requestedPackageLineItems: List[RequestedPackageLineItem]
    labelSpecification: LabelSpecification
    customsClearanceDetail: CustomsClearanceDetail
    shippingDocumentSpecification: ShippingDocumentSpecification
    emailNotificationDetail: Optional[EmailNotificationDetail]
    class Config:
        use_enum_values = True
        validate_all = True
    


from typing import List, Dict, Any, Optional
from pydantic import BaseModel
from .ServiceCommitMessage import ServiceCommitMessage
from .Destination import DestinationLocation, LastUpdatedDestinationAddress
from .OriginLocation import OriginLocation
from .DeliveryDetails import DeliveryDetails
from .SpecialHandling import SpecialHandling
from .DateAndTime import DateAndTime
from .LatestStatusDetail import LatestStatusDetail
from .RecipientInformation import RecipientInformation
from .ShipperInformation import ShipperInformation
from .AdditionalTrackingInfo import AdditionalTrackingInfo
from .TrackingNumberInfo import TrackingNumberInfo
from .Window import EstimatedDeliveryTimeWindow, StandardTransitTimeWindow
from .ServiceDetail import ServiceDetail
from .ScanEvent import ScanEvent
from .PackageDetails import PackageDetails
from .ShipmentDetails import ShipmentDetails

class TrackResult(BaseModel):
    trackingNumberInfo: TrackingNumberInfo
    additionalTrackingInfo: Optional[AdditionalTrackingInfo] = None
    shipperInformation: Optional[ShipperInformation] = None
    recipientInformation: Optional[RecipientInformation] = None
    latestStatusDetail: Optional[LatestStatusDetail] = None
    dateAndTimes: Optional[List[DateAndTime]] = None
    availableImages: Optional[List] = None
    specialHandlings: Optional[List[SpecialHandling]] = None
    packageDetails: Optional[PackageDetails] = None
    shipmentDetails: Optional[ShipmentDetails] = None
    scanEvents: Optional[List[ScanEvent]] = None
    availableNotifications: Optional[List[str]] = None
    deliveryDetails: Optional[DeliveryDetails] = None
    originLocation: Optional[OriginLocation] = None
    destinationLocation: Optional[DestinationLocation] = None
    lastUpdatedDestinationAddress: Optional[LastUpdatedDestinationAddress] = None
    serviceCommitMessage: Optional[ServiceCommitMessage] = None
    serviceDetail: Optional[ServiceDetail] = None
    standardTransitTimeWindow: Optional[StandardTransitTimeWindow] = None
    estimatedDeliveryTimeWindow: Optional[EstimatedDeliveryTimeWindow] = None
    goodsClassificationCode: Optional[str] = None
    returnDetail: Optional[Dict[str, Any]]

class CompleteTrackResult(BaseModel):
    trackingNumber: str
    trackResults: List[TrackResult]

class Output(BaseModel):
    completeTrackResults: List[CompleteTrackResult]
    
class TrackResponse(BaseModel):
    transactionId: str
    output: Optional[Output] = None


from typing import List
from .UploadDocumentReferenceDetail import UploadDocumentReferenceDetail
from pydantic import BaseModel


class CompletedEtdDetail(BaseModel):
    folderId: str
    type: str
    uploadDocumentReferenceDetails: List[UploadDocumentReferenceDetail]
    
    


from typing import Optional

from pydantic import BaseModel


class Address(BaseModel):
    city: str
    stateOrProvinceCode: Optional[str]
    countryCode: str
    residential: bool
    countryName: str


from typing import List

from .WeightAndDimensions import WeightAndDimensions
from pydantic import BaseModel

class PackagingDescription(BaseModel):
    type: str
    description: str

class PackageDetails(BaseModel):
    packagingDescription: PackagingDescription
    sequenceNumber: str
    count: str
    weightAndDimensions: WeightAndDimensions
    packageContent: List


from pydantic import BaseModel
from typing import List, Optional
from .DeliveryOptionEligiblityDetail import DeliveryOptionEligibilityDetail


class DeliveryDetails(BaseModel):
    deliveryAttempts: str
    deliveryOptionEligibilityDetails: List[DeliveryOptionEligibilityDetail]
    destinationServiceArea: Optional[str]


from typing import List
from .GenerationDetail import GenerationDetail
from pydantic import BaseModel



class DocumentRequirements(BaseModel):
    requiredDocuments: List[str]
    generationDetails: List[GenerationDetail]
    prohibitedDocuments: List[str]


from pydantic import BaseModel
from .DelayDetail import DelayDetail
from .ScanLocation import ScanLocation
from typing import Optional

class LatestStatusDetail(BaseModel):
    code: str
    derivedCode: str
    statusByLocale: str
    description: str
    scanLocation: ScanLocation
    delayDetail: Optional[DelayDetail]


from typing import Optional
from pydantic import BaseModel, root_validator, validator

class Contact(BaseModel):
    personName: str = ""
    emailAddress: str = ""
    phoneExtension: str = ""
    companyName: str = ""
    phoneNumber: str
    
    
    @root_validator(allow_reuse=True)
    def person_name_and_company_name_required_validate(cls, data):
        if data["personName"] == None and data["companyName"] == None:
            raise ValueError("Either the companyName or personName is mandatory")
        return data
        
    @validator("personName", allow_reuse=True)
    def person_name_validate(cls, value):
        if value is not None and len(value) > 70:
            raise ValueError(
                "Please Max 70 Character For Person Name"
            )
        return value
            
    @validator("emailAddress", allow_reuse=True)
    def emailAddress_validate(cls, value):
        if value is not None and len(value) > 80:
            raise ValueError(
                "Please Max 80 Character For Email Address"
            )
        return value
            
    @validator("phoneExtension", allow_reuse=True)
    def phoneExtension_validate(cls, value):
        if value is not None and len(value) > 6:
            raise ValueError(
                "Please Max 6 Character For Phone Extension"
            )
        return value
    
    @validator("companyName", allow_reuse=True)
    def companyName_validate(cls, value):
        if value is not None and len(value) > 35:
            raise ValueError(
                "Please Max 35 Character For Company Name Extension"
            )
        return value
    
    @validator("phoneNumber", allow_reuse=True)
    def phoneNumber_validate(cls, value):
        if value is not None:
            if len(value) > 15:
                
                raise ValueError(
                    "Please Max 15 Character For Phone Number"
                )
            elif len(value) < 10:
                raise ValueError(
                    "Please Min. 15 Character For Phone Number"
                )
        return value


from typing import List, Optional
from pydantic import BaseModel

from .Part import Part

class ShipmentDocument(BaseModel):
    type: Optional[str]
    shippingDocumentDisposition: Optional[str]
    imageType: Optional[str]
    resolution: Optional[int]
    copiesToPrint: Optional[int]
    parts: Optional[List[Part]]
    contentKey: Optional[str]
    contentType: Optional[str]
    copiesToPrint: Optional[int]
    encodedLabel: Optional[str]
    trackingNumber: Optional[str]
    docType: Optional[str]


from typing import List
from pydantic import BaseModel


class BinaryBarcode(BaseModel):
    type: str
    value: str
    
class StringBarcode(BaseModel):
    type: str
    value: str
    
class Barcodes(BaseModel):
    binaryBarcodes: List[BinaryBarcode]
    stringBarcodes: List[StringBarcode]


from typing import List, Optional
from pydantic import BaseModel

class Dimension(BaseModel):
    length: int
    width: int
    height: int
    units: str

class WeightItem(BaseModel):
    value: str
    unit: str

class WeightAndDimensions(BaseModel):
    weight: List[WeightItem]
    dimensions: Optional[List[Dimension]]


from pydantic import BaseModel
from typing import List, Dict, Any
from .ShipmentDocument import ShipmentDocument
from .PieceResponse import PieceResponse
from .CompletedShipmentDetail import CompletedShipmentDetail

class TransactionShipment(BaseModel):
    masterTrackingNumber: str
    serviceType: str
    shipDatestamp: str
    serviceName: str
    shipmentDocuments: List[ShipmentDocument]
    pieceResponses: List[PieceResponse]
    shipmentAdvisoryDetails: Dict[str, Any]
    completedShipmentDetail: CompletedShipmentDetail
    serviceCategory: str


class Output(BaseModel):
    transactionShipments: List[TransactionShipment]


class LabelResponse(BaseModel):
    transactionId: str
    output: Output



from typing import Dict, Any, Optional
from .Address import Address
from pydantic import BaseModel

class ShipperInformation(BaseModel):
    contact: Optional[Dict[str, Any]] = None
    address: Address


from pydantic import BaseModel
from typing import Optional

from .DelayDetail import DelayDetail

from .ScanLocation import ScanLocation

class ScanEvent(BaseModel):
    date: str
    eventType: str
    eventDescription: str
    exceptionCode: str
    exceptionDescription: str
    scanLocation: ScanLocation
    locationId: Optional[str] = None
    locationType: str
    derivedStatusCode: str
    derivedStatus: str
    delayDetail: Optional[DelayDetail] = None


from pydantic import BaseModel
from typing import Optional

from .LocationContactAndAddress import LocationContactAndAddress

class DestinationLocation(BaseModel):
    locationContactAndAddress: LocationContactAndAddress
    locationType: str


class LastUpdatedDestinationAddress(BaseModel):
    city: str
    stateOrProvinceCode: Optional[str]
    countryCode: str
    residential: bool
    countryName: str


import setuptools
from setuptools import find_packages

from FedexSDK.__init__ import __version__

setuptools.setup(
    name="fedex-sdk",
    version=__version__,
    author="Esat Yılmaz",
    author_email="esatyilmaz3500@gmail.com",
    description="Fedex Label Create SDK",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
    install_requires=[
    ]
)



from pydantic import BaseModel
from typing import List

from .PackageDocument import PackageDocument

class PieceResponse(BaseModel):
    masterTrackingNumber: str
    trackingNumber: str
    additionalChargesDiscount: float
    netRateAmount: float
    netChargeAmount: float
    netDiscountAmount: float
    packageDocuments: List[PackageDocument]
    customerReferences: List
    codcollectionAmount: float
    baseRateAmount: float


from typing import List, Optional
from pydantic import BaseModel

from .Address import Address
from .Contact import Contact
from .Tin import Tin

class Recipient(BaseModel):
    address: Address
    contact: Contact
    tins: Optional[List[Tin]]
    deliveryInstructions: str = ""
    